@@ -6,7 +6,7 @@ from .. import bhf_signals
 from contextlib import suppress
 from inspect import isawaitable
 from ipaddress import ip_address
-from typing import Callable, Dict, Generator, Optional, Tuple, Type, TypeVar
+from typing import Awaitable, Callable, Dict, Generator, Optional, Tuple, Type, TypeVar, Union
 
 from playwright.async_api import (
     BrowserContext,
@@ -72,44 +72,52 @@ def _get_page_method_result(page: Page, pm: PageMethod):
 
 class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
     def __init__(self, crawler: Crawler) -> None:
+        settings = crawler.settings
         super().__init__(settings=crawler.settings, crawler=crawler)
-        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
         verify_installed_reactor("twisted.internet.asyncioreactor.AsyncioSelectorReactor")
         crawler.signals.connect(self._engine_started, signals.engine_started)
         self.stats = crawler.stats
 
-        self.browser_type: str = crawler.settings.get("PLAYWRIGHT_BROWSER_TYPE") or "chromium"
-        self.page_load_until: str = crawler.settings.get("PLAYWRIGHT_PAGE_LOAD_UNTIL") or "load"
-        self.max_pages_per_context: int = crawler.settings.getint(
+        # browser
+        self.browser_type: str = settings.get("PLAYWRIGHT_BROWSER_TYPE") or "chromium"
+        self.browser_launch_lock = asyncio.Lock()
+        self.launch_options: dict = settings.getdict("PLAYWRIGHT_LAUNCH_OPTIONS") or {}
+
+        # contexts
+        self.max_pages_per_context: int = settings.getint(
             "PLAYWRIGHT_MAX_PAGES_PER_CONTEXT"
         ) or crawler.settings.getint("CONCURRENT_REQUESTS")
-        self.launch_options: dict = crawler.settings.getdict("PLAYWRIGHT_LAUNCH_OPTIONS") or {}
-
-        self.default_navigation_timeout: Optional[float] = None
-        if "PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT" in crawler.settings:
-            with suppress(TypeError, ValueError):
-                self.default_navigation_timeout = float(
-                    crawler.settings.get("PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT")
-                )
-
-        if crawler.settings.get("PLAYWRIGHT_PROCESS_REQUEST_HEADERS"):
-            self.process_request_headers = load_object(
-                crawler.settings["PLAYWRIGHT_PROCESS_REQUEST_HEADERS"]
-            )
-        else:
-            self.process_request_headers = use_scrapy_headers
-
-        self.context_kwargs: dict = crawler.settings.getdict("PLAYWRIGHT_CONTEXTS")
+        self.context_launch_lock = asyncio.Lock()
+        self.context_kwargs: dict = settings.getdict("PLAYWRIGHT_CONTEXTS")
         self.contexts: Dict[str, BrowserContext] = {}
         self.context_semaphores: Dict[str, asyncio.Semaphore] = {
             'default': asyncio.Semaphore(value=self.max_pages_per_context)
         }
 
-        self.abort_request: Optional[Callable[[PlaywrightRequest], bool]] = None
-        if crawler.settings.get("PLAYWRIGHT_ABORT_REQUEST"):
-            self.abort_request = load_object(crawler.settings["PLAYWRIGHT_ABORT_REQUEST"])
+        self.default_navigation_timeout: Optional[float] = None
+        if "PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT" in settings:
+            with suppress(TypeError, ValueError):
+                self.default_navigation_timeout = float(
+                    crawler.settings.get("PLAYWRIGHT_DEFAULT_NAVIGATION_TIMEOUT")
+                )
+
+        # headers
+        if "PLAYWRIGHT_PROCESS_REQUEST_HEADERS" in settings:
+            if settings["PLAYWRIGHT_PROCESS_REQUEST_HEADERS"] is None:
+                self.process_request_headers = None
+            else:
+                self.process_request_headers = load_object(
+                    settings["PLAYWRIGHT_PROCESS_REQUEST_HEADERS"]
+                )
+        else:
+            self.process_request_headers = use_scrapy_headers
+
+        self.abort_request: Optional[Callable[[PlaywrightRequest], Union[Awaitable, bool]]] = None
+        if settings.get("PLAYWRIGHT_ABORT_REQUEST"):
+            self.abort_request = load_object(settings["PLAYWRIGHT_ABORT_REQUEST"])
 
         self.user_agent_rotation_idx = 0
+        self.page_load_until: str = settings.get("PLAYWRIGHT_PAGE_LOAD_UNTIL") or "load"
         self.playwright_user_agents: list = crawler.settings.get("PLAYWRIGHT_USER_AGENTS") or []
         crawler.signals.connect(self._ip_changed, bhf_signals.ip_changed)
 
